deferred class GRAPH[K, E->EDGES[K]]
	-- VERTICES + EDGES

inherit
	VERTICES_SET[K]
		redefine make
		end

feature {NONE} -- Representation

	edges: EDGES_LIST[K]  -- E

feature {NONE} -- Creation

	make is
		-- construye un grafo vacio
	do
		Precursor
		create edges.make (Current)
	end

	from_graph (other: GRAPH[K, EDGES[K]]) is
		-- facilita conversiones
	local
		ii: ITERATOR[EDGE[K]]
	do
		from
			ii := other.get_new_iterator_on_edges
		until
			ii.is_off
		loop
			put_edge (ii.item)
			ii.next
		end
	end

feature -- edge operations

	put_edge (e: EDGE[K]) is
		-- agrega `e' al grafo. Si ya existe, no hace nada.
	require
		is_valid_edge (e.source, e.target)
	do
		add_edge (e.source, e.target)
	ensure
		are_adjacents (e.source, e.target)
	end

	add_edge(v1, v2: VERTEX[K]) is
		-- agrega una arista entre `v1' y `v2'. Si ya existe, no hace nada.
	require
		is_valid_edge (v1, v2)
	deferred
	ensure
		are_adjacents(v1, v2)
	end

	remove_edge(v1, v2: VERTEX[K]) is
		-- quita la arista entre `v1' y `v2'
	deferred
	ensure
		not are_adjacents(v1, v2)
	end

	are_adjacents(v1, v2: VERTEX[K]): BOOLEAN is
		-- hay una arista entre `v1' y `v2'?
	do
		Result := edges.has (v1, v2)
	ensure
		Result implies is_valid_edge (v1, v2)
	end

	adjacents(v: VERTEX[K]): ITERATOR[VERTEX[K]] is
		-- { w : are_adjacents (v, w) }
	require
		has_vertex(v)
	do
		Result := edges.at (v)
	end

	valency (v: VERTEX[K]): INTEGER is
		-- cantidad de adjacents a `v'
	require
		has_vertex(v)
	do
		Result := edges.count (v)
	end

	get_new_iterator_on_edges: ITERATOR[EDGE[K]] is
		-- itera sobre las aristas del grafo
	do
		create {ITERATOR_EDGES[K]} Result.make (Current)
	end

feature -- Properties

	is_connected: BOOLEAN is
		-- es un grafo conexo ?
	local
		bfs: ITERATOR[VERTEX[K]]; c: INTEGER
	do
		from
			bfs := get_new_iterator_bfs(last_vertex_added)
		until
			bfs.is_off
		loop
			c := c + 1
			bfs.next
		end
		Result := c = count
	end

	has_cycle: BOOLEAN is
		-- contiene un ciclo ?
	local
		bfs: BFS_REPEATING[K]; visited: SET[VERTEX[K]]
	do
		-- TODO: solo anda para grafos conexos
		from
			create bfs.make (Current, last_vertex_added)
			Result := True
		until
			bfs.is_off or not Result
		loop
			if visited.has (bfs.item) then
				Result := False
			end
			visited.add (bfs.item)
			bfs.next
		end
	end

	is_tree: BOOLEAN is
		-- es un arbol?
	do
		Result := is_connected and not has_cycle
	end

	is_complete: BOOLEAN is
		-- es un grafo completo?
	local
		ii, jj: ITERATOR[VERTEX[K]]
	do
		Result := True
		from
			ii := get_new_iterator
		until
			ii.is_off or not Result
		loop
			from
				jj := get_new_iterator
			until
				jj.is_off or not Result
			loop
				Result := are_adjacents (ii.item, jj.item)
				jj.next
			end
			ii.next
		end
	end

	is_regular: BOOLEAN is
		-- es un grafo regular?
	local
		ii: ITERATOR[VERTEX[K]]
		v: INTEGER
	do
		Result := True
		from
			ii := get_new_iterator
			v := valency (ii.item)
		until
			ii.is_off or not Result
		loop
			Result := v = valency (ii.item)
			ii.next
		end
	end

feature -- Algorithms

	get_new_iterator_bfs (init: VERTEX[K]): BFS[K] is
		-- itera sobre los vertices en `bfs' comenzando en `init'
	do
		create Result.make (Current, init)
	end

	get_new_iterator_dfs (init: VERTEX[K]): DFS[K] is
		-- itera sobre los vertices en `dfs' comenzando en `init'
	do
		create Result.make (Current, init)
	end

	get_new_iterator_greedy: ITERATOR[VERTEX[K]] is
		-- itera sobre los vertices en `greedy'
	do
		create {ITERATOR_GREEDY[K]} Result.make (Current)
	end

	couloring_number_greedy: INTEGER is
		-- corre ``greedy'' sobre el grafo
	local
		color: ARRAY[INTEGER]
		colors: ARRAY[BOOLEAN]
		ii, adj: ITERATOR[VERTEX[K]]
		i: INTEGER
	do
		from
			ii := get_new_iterator_greedy
		until
			ii.is_off
		loop
			-- obtiene los colores de los vecinos
			from
				create colors.make (1, count)
				adj := adjacents (ii.item)
			until
				adj.is_off
			loop
				if color.item (adj.item.index) > 0 then
					colors.put (True, adj.item.index)
				end
				adj.next
			end
			-- busca el primero libre
			from
				i := 1
			until
				color @ i = 0
			loop
				i := i + 1
			end
			Result := Result.max (i)
			color.put (i, ii.item.index)
			ii.next
		end
	end

	is_bipartite: BOOLEAN is
		-- es un grafo bipartito?
	local
		color: ARRAY[INTEGER]
		bfs: BFS_REPEATING[K]
		next: VERTEX[K]
	do
		create color.make (1, count)
		-- colorear el primero
		next := last_vertex_added
		color.put (1, next.index)
		from
			create bfs.make (Current, next)
			-- el primero ya esta coloreado
			bfs.next
		until
			bfs.is_off or not Result
		loop
			if color.item (bfs.parent.index) = color.item(bfs.item.index) then
				-- hay un ciclo
				Result := False
			end
			color.put (bfs.item.index, -color.item(bfs.parent.index))
			bfs.next
		end
	end

feature -- Paths

	exists_path(v1, v2: VERTEX[K]): BOOLEAN is
		-- hay un camino desde `v1' a `v2'
	require
		has_vertex(v1)
	local
		bfs: ITERATOR[VERTEX[K]]
	do
		from
			bfs := get_new_iterator_bfs(v1)
		until
			bfs.is_off or else bfs.item = v2
		loop
			bfs.next
		end
		Result := not bfs.is_off
	end

	distance (v1, v2: VERTEX[K]): INTEGER is
		-- longitud del camino minimo desde `v1' a `v2'
	require
		exists_path (v1, v2)
	local
		bfs: ITERATOR[VERTEX[K]]
	do
		from
			bfs := get_new_iterator_bfs(v1)
		until
			bfs.is_off or else bfs.item = v2
		loop
			Result := Result + 1
			bfs.next
		end
	end

	eulerian_path (v1, v2: VERTEX[K]): BOOLEAN is
		-- hay un recorrido euleriano de `v1' a `v2'
	require
		has_vertex (v1)
		has_vertex (v1)
	do
		Result := is_connected
		if v1 = v2 then
			Result := Result and (valency (v1) \\ 2 = 0)
		else
			Result := Result and (valency (v1) \\ 2 = 1) and (valency (v2) \\ 2 = 1)
		end
		from
			ii := get_new_iterator
		until
			ii.is_off or not Result
		loop
			if ii.item /= v1 and ii.item /= v2 then
				Result := valency (ii.item) = 2
			end
			ii.next
		end
	ensure
		Result implies is_connected
		Result and v1 = v2 implies valency (v1) mod 2 = 0
		Result and v1 /= v2 implies valency (v1) mod 2 = 1 and valency (v2) mod 2 = 1
	end

feature -- Validation

	is_valid_edge (v1, v2: VERTEX[K]): BOOLEAN is
		-- es (`v1', `v2') una arista valida? Esto permite establecer restricciones. 
		-- Por ejemplo, en un grafo bipartito no se pueden unir dos v\'ertices de 
		-- la misma parte. En un \'arbol, no se puede agregar una arista si genera 
		-- un ciclo.
	do
		Result := has_vertex(v1) and has_vertex(v2)
	ensure
		Result implies has_vertex (v1) and has_vertex (v2)
	end

feature -- Output

	output_header: STRING is
	deferred
	end

	output_vertex (v: VERTEX[K]): STRING is
	require
		has_vertex (v)
	do
		Result := v.index.to_string
	end

	output_unweighted_edge (v1, v2: VERTEX[K]): STRING is
	require
		are_adjacents (v1, v2)
	deferred
	end

	output_edge (v1, v2: VERTEX[K]): STRING is
	require
		are_adjacents (v1, v2)
	do
		Result := output_unweighted_edge (v1, v2)
	end

end -- class GRAPH[K, E]
