deferred class WEIGHTED_GRAPH[W->DOUBLE,K,E->EDGES[K]]

inherit
	GRAPH[K,E]
		rename add_edge as add_edge_old, remove_edge as remove_edge_old
		redefine make, get_new_iterator_on_edges, put_edge, from_graph, add_vertex, output_edge
		end

feature {NONE}
	-- Creation

	make is
	do
		Precursor
		create weights.make(1, 1, 1, 1)
	end

	from_graph(other: WEIGHTED_GRAPH[W,K,EDGES[K]]) is
	local
		ii: ITERATOR[WEIGHTED_EDGE[W,K]]
	do
		from
			ii := other.get_new_iterator_on_edges
		until
			ii.is_off
		loop
			put_edge(ii.item)
			ii.next
		end
	end

feature {NONE}
	-- Representation

	weights: ARRAY2[W]
			-- pesos

feature -- operations

	weight(v1, v2: VERTEX[K]): W is
		-- peso de la arista entre `v1' y `v2'.
	require
		has_vertex(v1)
		has_vertex(v2)
		are_adjacents(v1, v2)
	do
		Result := weights.item(v1.index, v2.index)
	end

	put_edge(e: WEIGHTED_EDGE[W,K]) is
	do
		add_edge(e.source, e.target, e.weight)
	end

	add_edge(v1, v2: VERTEX[K]; w: W) is
	do
		if not are_adjacents(v1, v2) then
			change
		end
		add_edge_old(v1, v2)
		if weight(v1, v2) /= w then
			change
		end
	ensure
		weight(v1, v2) = w
	end

	remove_edge(v1, v2: VERTEX[K]) is
	do
		if are_adjacents(v1, v2) then
			change
		end
		remove_edge_old(v1, v2)
	ensure
		not are_adjacents(v1, v2)
	end

	get_new_iterator_on_edges: ITERATOR[WEIGHTED_EDGE[W,K]] is
	do
		create {ITERATOR_WEIGHTED_EDGES[W,K]} Result.make(Current)
	end

	get_new_iterator_on_ordered_edges: ITERATOR[WEIGHTED_EDGE[W,K]] is
	do
		create {ITERATOR_ORDERED_EDGES[W,K]} Result.make(Current)
	end

feature -- operations

	non_negative_weights: BOOLEAN is
	local
		ii: ITERATOR[WEIGHTED_EDGE[W, K]]
	do
		from
			Result := True
			ii := get_new_iterator_on_edges
		until
			ii.is_off or not Result
		loop
			Result := ii.item.weight >= 0
			ii.next
		end
	end

feature -- paths

	shortest_path(source, target: VERTEX[K]): W is
		-- Calcula el camino mas corto de `v1' a `v2'
	require
		exists_path(source, target)
		non_negative_weights
	do
		-- vertices cuya distancia minima desde `x' es conocida
		if has_changed then
			initialize_paths
		end
		if not get_known (source, target) then
			dijkstra (source, target, True)
		end
		Result := get_distance(source, target)
	end

	all_paths is
		-- Calcula el camino mas corto entre cada par de vertices
	do
		if has_changed then
			initialize_paths
		end
		floyd
	end

	all_paths_from_vertex(v: VERTEX[K]) is
		-- Calcula el camino mas corto de `v' a todos los vertices
	do
		if has_changed then
			initialize_paths
		end
		dijkstra (v, v, False)
	end

	kruscal: like Current is
		-- obtiene un arbol de expansion minimal usando Kruscal
	require
		is_connected
	local
		ii: ITERATOR[WEIGHTED_EDGE[W,K]]; uf: FOREST; v1, v2: VERTEX[K]
	do
		create Result.make
		Result.from_collection(Current)
		from
			create uf.make(count)
			ii := get_new_iterator_on_ordered_edges
		until
			ii.is_off or uf.count = 1
		loop
			v1 := ii.item.source
			v2 := ii.item.target
			if uf.are_connected(v1.index, v2.index) then
				Result.add_edge(v1, v2, ii.item.weight)
				uf.union(v1.index, v2.index)
			end
			ii.next
		end
		check
			uf.count = 1
		end
	ensure
		Result.count = count
		Result.is_tree
	end

	prim: like Current is
		-- obtiene un arbol de expansion minimal usando Prim
	require
		is_connected
	do
	ensure
		Result.count = count
		Result.is_tree
	end

feature {NONE}
	-- to calculate paths

	distances: ARRAY2[W] -- distancias ya calculadas

	get_distance (v1, v2: VERTEX[K]): W is
	require
		get_known (v1, v2)
	do
		Result := distances.item (v1.index, v2.index)
	end

	put_distance (w: W; v1, v2: VERTEX[K]) is
	do
		distances.put (w, v1.index, v2.index)
		put_known (True, v1, v2)
	ensure
		get_distance (v1, v2) = w
		get_known (v1, v2)
	end

	knowns: ARRAY2[BOOLEAN] -- cuales estan ya calculadas

	get_known (v1, v2: VERTEX[K]): BOOLEAN is
	do
		Result := knowns.item (v1.index, v2.index)
	end

	put_known (b: BOOLEAN; v1, v2: VERTEX[K]) is
	do
		knowns.put (b, v1.index, v2.index)
	end

	has_changed: BOOLEAN
			-- a cambiado los pesos en el grafo desde la ultima vez
			-- que calculamos los shortest paths?

	change is
		-- ocurrio algun cambio en los pesos del grafo
	do
		has_changed := True
	end

	initialize_paths is
		-- inicializa el calculo de paths
	local
		ii: ITERATOR[VERTEX[K]]
	do
		has_changed := False
		if distances = Void then
			create distances.make (1, count, 1, count)
			create knowns.make (1, count, 1, count)
		end
		knowns.clear_all
		from
			ii := get_new_iterator
		until
			ii.is_off
		loop
			put_distance (0, ii.item, ii.item)
			ii.next
		end
	ensure
		not has_changed
	end

feature {VERTEX}
	-- Internal

	add_vertex(v: VERTEX[K]) is
	do
		Precursor(v)
		on_change
	end

	on_change is
	do
		if count > 0 then
			weights.resize(1, count, 1, count)
		end
	end

feature {NONE} -- Internal

	dijkstra (source, target: VERTEX[K]; stop: BOOLEAN) is
		-- Corre dijkstra para calcular el camino mas corto de `source' a `target'
	require
		exists_path(source, target)
		non_negative_weights
		not has_changed
	local
		closest: SET[VERTEX[K]] -- vertices cuya distancia minima desde `source' es conocida
		min: W
		min_vertex: VERTEX[K]
		ii, jj: ITERATOR[VERTEX[K]]
		u, v: VERTEX[K]
	do
		create closest.make
		from
			if true then
			from
				ii := get_new_iterator
			until
				ii.is_off
			loop
				if get_known(source, ii.item) then
					-- agrego a `closest' todos los vertices
					-- cuya distancia ya conozco
					closest.add(ii.item)
				end
				ii.next
			end
			else
			closest.add (source)
			end
		until
			stop and closest.has(target) or closest.count = count
		loop
			min_vertex := Void
			from
				ii := closest.get_new_iterator
			until
				ii.is_off
			loop
				from
					u := ii.item
					jj := adjacents(u)
				until
					jj.is_off
				loop
					v := jj.item
					if not closest.has(v) then
						-- para todo par (u, v) tal que
						-- (u, v) es una arista
						-- `u' en `closest' y `v' fuera de `closest'
						if get_known (source, u) and then (min_vertex = Void or else min > get_distance(source, u) + weight(u, v)) then
							min := get_distance(source, u) + weight(u, v)
							min_vertex := v
						end
					end
					jj.next
				end
				ii.next
			end
			closest.add(min_vertex)
			put_distance(min, source, min_vertex)
		end
	ensure
		get_known (source, target)
	end

feature -- Output

	output_edge (v1, v2: VERTEX[K]): STRING is
	do
		Result := Precursor (v1, v2) + " [label = " + weight (v1, v2).to_string + "]"
	end

end -- class WEIGHTED_GRAPH
